/*
 * LocationParser.java
 */
package de.holzem.refbuilder;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;
import java.util.Stack;
import java.util.StringTokenizer;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.Task;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

/**
 * LocationParser
 */
public class LocationParser
{
	Task task;
	Index index;
	String[] files;
	File dir;

	/**
	 * Constructor
	 * 
	 * @param task
	 *          for logging purposes
	 * @param dir
	 *          directory path for the files
	 * @param files
	 *          the files to scan
	 */
	public LocationParser(Task task, File dir, String[] files)
	{
		this.task = task;
		this.dir = dir;
		this.files = files;
		this.index = new Index();
	}

	/**
	 * parse list of files for Locations
	 */
	public void parse()
	{
		task.log("  Parsing for Location", Project.MSG_INFO);
		for (int i = 0; i < files.length; i++) {
			String file = files[i];
			if (file.equals("reiseberichte\\schottlandkarten\\index.xml")) {
				continue;
			}
			String path = dir + "\\" + file;
			//task.log("  Parsing for Location: " + file);
			parseFile(path, file);
		}
	}

	/**
	 * parse one file for Locations
	 * 
	 * @param path
	 *          path to file
	 * @param file
	 *          filename for website-relative reference
	 */
	private void parseFile(String path, String file)
	{
		InputStream is = null;
		try {
			is = new FileInputStream(path);
		} catch (FileNotFoundException e) {
			task.log(e.getClass().getName() + ":\n" + e.getMessage());
			return;
		}
		try {
			SAXParserFactory spf = SAXParserFactory.newInstance();
			// be sure that NO validation takes place
			spf.setValidating(false);
			spf.setFeature("http://xml.org/sax/features/namespaces", false);
			spf.setFeature("http://xml.org/sax/features/validation", false);
			spf.setFeature(
					"http://apache.org/xml/features/nonvalidating/load-external-dtd",
					false);
			SAXParser saxParser = spf.newSAXParser();
			LocationHandler handler = new LocationHandler(task, file, index);
			saxParser.parse(is, handler);
		} catch (Exception e) {
			task
					.log(e.getClass().getName() + ":\n" + e.getMessage(), Project.MSG_ERR);
		}
	}

	/**
	 * @return the parsed Index
	 */
	public Index getIndex()
	{
		return index;
	}

	/**
	 * LocationHandler: a SAXHandler for Locations
	 */
	private static class LocationHandler extends DefaultHandler
	{
		Map isIncluded = new HashMap(101);
		Task task = null;
		String file = null;
		Stack stack = null;
		StringBuffer value = null;
		String title = null;
		Index index = null;
		String map = null;
		String maptype = null;

		/**
		 * LocationHandler picks up location definitions in the website's XML files.
		 * The following tags are parsed:
		 * <ul>
		 * <li>area-tags of maps. The entry-name is specified by the href's part
		 * after the hash</li>
		 * <li>ortsindex. The entry-name is specified by the attribute entry</li>
		 * </ul>
		 * 
		 * @param task
		 *          for logging
		 * @param file
		 *          source of the references
		 * @param index
		 *          index to add the references to
		 */
		LocationHandler(Task task, String file, Index index)
		{
			this.task = task;
			this.file = "/" + file.replace('\\', '/').replace(".xml", ".html");
			this.stack = new Stack();
			this.value = new StringBuffer();
			this.index = index;
		}

		/**
		 * Find the title of the document. It has the path document/header/title.
		 * For this purpose the Stack's toString() method is used.
		 * 
		 * @see org.xml.sax.ContentHandler#endElement(java.lang.String,
		 *      java.lang.String, java.lang.String)
		 */
		public void endElement(String uri, String localName, String qName)
				throws SAXException
		{
			// the relevant title is to be found
			if (stack.toString().equals("[document, header, title]")) {
				title = value.toString().trim().replace("\"", "");
			}
			String element = (String)stack.pop();
		}

		/**
		 * Parse for ortsindex, map and area-tags
		 * 
		 * @see org.xml.sax.ContentHandler#startElement(java.lang.String,
		 *      java.lang.String, java.lang.String, org.xml.sax.Attributes)
		 */
		public void startElement(String uri, String localName, String qName,
				Attributes attributes) throws SAXException
		{
			// parse ortsindex-tag for attribute entry
			if (qName.equals("ortsindex")) {
				String entry = attributes.getValue("entry").trim();
				String name = splitEntry(entry);
				index
						.add(name, entry, file, "Reisebericht: " + title);
			}
			// parse MAP-tag for the type (basis, 1994-*)
			if (qName.equalsIgnoreCase("MAP")) {
				map = attributes.getValue("NAME");
				maptype = map.substring(map.lastIndexOf('-') + 1);
			}
			// parse AREA-tag
			if (qName.equalsIgnoreCase("AREA")) {
				String href = attributes.getValue("HREF").trim();
				String entry = href.substring(href.lastIndexOf('#') + 1);
				boolean doAdd = false;
				if (isIncluded.get(entry) == null) {
					isIncluded.put(entry, entry);
					doAdd = true;
				}
				if (doAdd) {
					String name = splitEntry(entry);
					String titleprefix = "Karte ";
					if (!maptype.equals("basis")) {
						titleprefix = titleprefix + maptype + " ";
					}
					index.add(name, entry, file + "#" + map, titleprefix + title);
				}
			}
			stack.push(qName);
			value.setLength(0);
		}

		/**
		 * Collect Strings for tags. It is only needed to find the entire title-tag
		 * in the document's header.
		 * 
		 * @see org.xml.sax.ContentHandler#characters(char[], int, int)
		 */
		public void characters(char[] ch, int start, int length)
				throws SAXException
		{
			String trimValue = new StringBuffer().append(ch, start, length)
					.toString().trim();
			value.append(" ").append(trimValue);
		}

		/**
		 * Entries are without spaces to allow the locating in another file
		 * 
		 * @param entry
		 * @return
		 */
		private String splitEntry(String entry)
		{
			final String upper = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
			StringBuffer entry2 = new StringBuffer();
			StringTokenizer st = new StringTokenizer(entry, upper, true);
			while (st.hasMoreElements()) {
				String token = st.nextToken();
				if (token.length() == 1) {
					if (upper.indexOf(token) >= 0) {
						entry2.append(" ");
					}
				}
				entry2.append(token);
			}
			return entry2.toString().trim();
		}
	}
}